/*
ATtiny45V

SELFPRGEN=0
RSTDISBL=0
DWEN=0
SPIEN=1
WDTON=0
EESAVE=0
BODLEVEL=Brown-out detection at VCC=2.7 V
CKDIV8=0
CKOUT=0
SUT_CKSEL=Int. RC Osc. 8 MHz; Start-up time PWRDWN/RESET: 6 CK/14 CK + 4 ms

LOW/HIGH/EXTENDED = 0xD2/0xDD/0xFF
*/

#define F_CPU 8000000UL // Taktfrequenz 8 MHz

#include <avr/io.h>
#include <avr/interrupt.h>

/**************
*             *
*  Variablen  *
*             *
**************/

volatile unsigned char PWMValue;
volatile unsigned char Dimmfaktor;  // bestimmt die Helligkeit (Dimmen)
volatile unsigned char DimmEnable;  // 1 = PWM luft; 0 = PWM-Ausgang AUS

/******************
*                 *
*  Hauptprogramm  *
*                 *
******************/

int main(void)
{    
  // Timer 0 einstellen

  TCCR0B = (5<<CS00);  // Teiler /1024
  Dimmfaktor = 101;
  OCR0A = Dimmfaktor;
  OCR0B = 10;

  // Timer 1 einstellen

  OCR1A = 2;  // PWM-Ausgang zunchst aus
  OCR1B = 10;  // dazwischen mit PWM synchronisierten Interrupt auslsen
  OCR1C = 0xFF;  // volle Auflsung
  TCCR1 = (1<<PWM1A) | (2<<COM1A0) | (1<<CS10);  // fast PWM (non-inv.)

  TIMSK = (1<<TOIE0) | (1<<OCIE0A) | (1<<OCIE0B) | (1<<OCIE1B);  // IRQs einschalten

  // Ports einstellen

  PORTB = 0b00100000;
  DDRB  = 0b00011111;
  //          |||||+- 0[5]:PB0
  //          ||||+-- 1[6]:PWMout (OC1A)
  //          |||+--- 2[7]:PB2
  //          ||+---- 3[2]:PB3
  //          |+----- 4[3]:PB4
  //          +------ 5[1]:Reset

  // Variablen einstellen

  PWMValue = 100;
  DimmEnable = 1;

  sei();

  // Hauptschleife

  while (1);
}

/******************************
*                             *
*  Timer 0 Compare Match A    *
*  PWM-Dimmer, Unterbrechung  *
*                             *
******************************/

ISR(TIM0_COMPA_vect)
{
  PORTB &= ~(1<<PB2);

  if (OCR0A < 255)  // falls gedimmt werden soll
  {
    DimmEnable = 0;  // Dunkelphase der Dimmung
    OCR1A = 0;  // PWM abschalten
  }

  TIFR |= (1<<OCF1B);  // Interrupt-Flag lschen !!!
}

/**************************************
*                                     *
*  Timer/Counter0 Overflow            *
*  PWM-Dimmer, Unterbrechung beenden  *
*                                     *
**************************************/

ISR(TIM0_OVF_vect)
{
  OCR0A = Dimmfaktor;  // ggf. genderten Dimmwert bernehmen

  if (DimmEnable == 0)  // falls PWM aus ist
  {
    TIFR |= (1<<OCF1B);  // Interrupt-Flag lschen !!!
    TIMSK |= (1<<OCIE1B);  // Timer-1-Compare-B-IRQ einschalten

    OCR1A = PWMValue;  // PWM wieder einschalten
    DimmEnable = 1;
  }

  PORTB |= (1<<PB2);
}

/***********************************
*                                  *
*  Timer/Counter0 Compare Match B  *
*  Zeitbasis fr Abfrage           *
*                                  *
************************************/

ISR(TIM0_COMPB_vect)
{
  OCR0B += 10;

  PORTB |= (1<<PB0);
  TIFR |= (1<<OCF1B);  // Interrupt-Flag lschen !!!
  PORTB &= ~(1<<PB0);
}

/****************************************
*                                       *
*  Timer/Counter1 Compare Match B       *
*  Synchronisierung mit der Regler-PWM  *
*                                       *
****************************************/

ISR(TIM1_COMPB_vect)
{
  PORTB |= (1<<PB3);
  TIMSK &= ~(1<<OCIE1B);  // Timer-1-Compare-B-IRQ abschalten
  PORTB &= ~(1<<PB3);
}
